'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = mixer;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _lodashObjectForOwn = require('lodash/object/forOwn');

var _lodashObjectForOwn2 = _interopRequireDefault(_lodashObjectForOwn);

var _lodashObjectForIn = require('lodash/object/forIn');

var _lodashObjectForIn2 = _interopRequireDefault(_lodashObjectForIn);

var _lodashLangCloneDeep = require('lodash/lang/cloneDeep');

var _lodashLangCloneDeep2 = _interopRequireDefault(_lodashLangCloneDeep);

var _lodashLangIsObject = require('lodash/lang/isObject');

var _lodashLangIsObject2 = _interopRequireDefault(_lodashLangIsObject);

var mergeSourceToTarget = undefined;

/**
 * Factory for creating mixin functions of all kinds.
 *
 * @param {Object} opts
 * @param {Function} opts.filter Function which filters value and key.
 * @param {Boolean} opts.chain Loop through prototype properties too.
 * @param {Boolean} opts.deep Deep looping through the nested properties.
 * @param {Boolean} opts.clone Do not mutate the target object. Clone it instead.
 */

function mixer() {
  var opts = arguments[0] === undefined ? {} : arguments[0];

  /**
   * Combine properties from the passed objects into target. This method mutates target,
   * if you want to create a new Object pass an empty object as first param.
   *
   * @param {Object} target Target Object
   * @param {...Object} objects Objects to be combined (0...n objects).
   * @return {Object} The mixed object.
   */
  return function (target) {
    for (var _len = arguments.length, objects = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      objects[_key - 1] = arguments[_key];
    }

    var loop = opts.chain ? _lodashObjectForIn2['default'] : _lodashObjectForOwn2['default'];
    var iteratee = function iteratee(val, key) {
      if (opts.filter && !opts.filter(val, key)) {
        return;
      }

      this[key] = opts.deep ? mergeSourceToTarget(this[key], val, opts) : val;
    };

    target = opts.clone ? (0, _lodashLangCloneDeep2['default'])(target) : target;

    objects.forEach(function (obj) {
      loop(obj, iteratee, target);
    });

    return target;
  };
}

mergeSourceToTarget = function (targetVal, srcVal, opts) {
  var isTargObj = (0, _lodashLangIsObject2['default'])(targetVal);
  var isSrcObj = (0, _lodashLangIsObject2['default'])(srcVal);

  if (isTargObj && !isSrcObj) {
    return targetVal;
  } else if (isTargObj && isSrcObj) {
    /**
     * Inception, deep merge objects
     */
    return mixer(opts)(targetVal, srcVal);
  } else if (isSrcObj) {
    /**
     * Target is not object, but source is. Make target an object too.
     */
    return mergeSourceToTarget({}, srcVal, opts);
  }

  /**
   * Make sure arrays, regexp, date, numbers are cloned
   */
  return (0, _lodashLangCloneDeep2['default'])(srcVal);
};
module.exports = exports['default'];